<?php

/**
 * Function that triggers when our Ad is saved
 * In this function we schedule the end of this ad to clear the transient and set the status of Ad 
 * @param  number $post_id 
 * @param  object $post    
 * @return mixed          
 */
function swa_save_ad( $post_id, $post ){

	$slug = 'swa';

	if ( $slug != $post->post_type ) {
        return;
    }

    $format = 'd-m-Y H:i';
    $post_date = get_the_date( $format, $post_id );
    $datetime = DateTime::createFromFormat( $format, $post_date );
	$timestamp = $datetime->getTimestamp();

	// Get the current Server Time
    $current_time = time();

    // Get the Current WordPress time
    $wp_current_time = current_time( 'timestamp' );

    // Get GMT offset
    $offset = get_option('gmt_offset');

    // Get Timezone string
    $timezone = get_option('timezone_string'); 
    $difference = 0;

    if( $timezone ) {
		date_default_timezone_set($timezone);
	} else {
        // Get the difference between. Difference will be minus if that time did not happen yet (such as GMT+0 and GMT+6)
        $difference = $wp_current_time - $current_time;
    }

    // Multiply by -1 to get + so NY != -6 --> == +6
    $the_end_time = (-1) * $difference + $timestamp;

    // Get the duration of Ad
    $duration_of_ad = 0;

    if( isset( $_POST['duration'] ) ) {

         $duration_of_ad = $_POST['duration'];

    }

    $duration_of_ad_timestamp = $duration_of_ad * DAY_IN_SECONDS;

    $the_end_time += $duration_of_ad_timestamp;

    delete_transient( 'swa_ads' );

    // Don't schedule if the end time is in the past
    if( $wp_current_time > $the_end_time ) {

        return $post_id;

    }

    // Post Status is 'Ended', publish it back
    if( $post->post_status == 'ended' ){
    	swa_set_status( $post_id, 'swa', 'publish', 'ended' );
    }

    $scheduled_end_of_ad = wp_next_scheduled( 'swa_ad_ended', array( $post_id ) );
    
    // If already scheduled, remove it
    if( $scheduled_end_of_ad ) {
        wp_unschedule_event( $scheduled_end_of_ad, 'swa_ad_ended', array( $post_id ) );
    }
    
    // Schedule the end
    wp_schedule_single_event( $the_end_time, 'swa_ad_ended', array( $post_id ) );
}

/**
 * Set status for a post
 * @param  number $post_id     ID of the post for which we want to set the status
 * @param  string $post_type   Post Type 
 * @param  string $status      New status we want to set
 * @param  string $from_status Old status, from which we will set
 * @return boolean             True/False
 */
function swa_set_status( $post_id, $post_type, $status, $from_status = 'publish'){
	if( get_post_type( $post_id ) == $post_type ){
		 global $wpdb;
		 $update_status = $wpdb->update( 
			$wpdb->posts, 
			array( 
				'post_status' =>  $status,	// string 
			), 
			array( 'ID' => $post_id, 'post_type' => $post_type, 'post_status' => $from_status ), 
			array( 
				'%s',	// value1 
			), 
			array( '%d', '%s' ) 
		);

		if( $update_status ){

			wp_transition_post_status( $status, $from_status, get_post($post_id) );
			return true;

		}
		return false;
	}
}

/**
 * Function triggered when the ad has ended
 * @param  number $post_id 
 * @return void
 */
function swa_ad_ended( $post_id ) {

	swa_set_status( $post_id, 'swa', 'ended' );

	delete_transient( 'swa_ads' );

}

/**
 * Display the impressions in the column
 * @param  number $post_id    
 * @param  string $column_key Column key
 * @return number             
 */
function swa_impressions( $post_id, $column_key ) {
    $impressions = get_post_meta( $post_id, $column_key, true );
    if( ! $impressions ) {
        $impressions = 0;
    }
    echo $impressions;
}

/**
 * Display the impressions in the column
 * @param  number $post_id    
 * @param  string $column_key Column key
 * @return number             
 */
function swa_clicks( $post_id, $column_key ) {
    $clicks = get_post_meta( $post_id, $column_key, true );
    if( ! $clicks ) {
        $clicks = 0;
    }
    echo $clicks;
}

/**
 * Display the impressions in the column
 * @param  number $post_id    
 * @param  string $column_key Column key
 * @return number             
 */
function swa_ctr( $post_id, $column_key ) {
    $impressions = get_post_meta( $post_id, '_impressions', true );
    $clicks = get_post_meta( $post_id, '_clicks', true );
    
    if( ! $impressions ) {
        $impressions = 0;
    }

    if( ! $clicks ) {
        $clicks = 0;
    }

    $ctr = 0;

    if( $clicks > 0 && $impressions > 0 ) {
        $ctr = (float)( $clicks / $impressions );
    }

    echo number_format($ctr, 2, '.', '') . '%';
}

/**
 * Click Ad registration
 * @return void 
 */
function swa_ad_click() {
    $id = 0;

    if( isset( $_POST['id'] ) ) {
        $id = (int) $_POST['id'];
    }

    $clicks = (int) get_post_meta( $id, '_clicks', true );

    if( ! $clicks ) {
        $clicks = 0;
    }

    $clicks++;
    update_post_meta( $id, '_clicks', $clicks );

    wp_die();
}