<?php
/**
 * Plugin Name: Simple WP Ads
 * Plugin URI: https://www.ibenic.com/simple-wordpress-advertising-plugin-the-introduction/
 * Description: Plugin made from the tutorial on ibenic.com
 * Version: 1.0
 * Author: Igor Benic
 * Author URI: https://ibenic.com
 *
 * Text Domain: swa
 * Domain Path: /languages/
 *
 * @package Simple WP Ads
 * @author Igor Benic
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; 
}

define( 'SWA_ROOT', plugin_dir_path( __FILE__ ) );
define( 'SWA_URI', plugin_dir_url( __FILE__ ) );

/**
 * Main Class
 */
class Simple_WP_Ads {

	/**
	 * The single instance of the class.
	 */
	protected static $_instance = null;

	/**
	 * Ads to be rendered
	 * @var array
	 */
	protected $ads = array();

	/**
	 * Getting the static instance
	 * @return object 
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Return ads
	 * @return mixed 
	 */
	public function get_ads(){
		return $this->ads;
	}

	/**
	 * Sets ads
	 * @param mixed $ads 
	 */
	public function set_ads( $ads ){
		$this->ads = $ads;
	}

	/**
	 * Check if any ad is in position
	 * @param  string $position the position to check
	 * @return mixed           
	 */
	public function has_ad_position( $position ){
		if( ! is_array( $this->ads ) ) {
			return false;
		}

		if( isset( $this->ads[ $position ] ) && count( $this->ads[ $position ] ) > 0 ) {
			return true;
		}

		return false;
	}

	/**
	 * Load dependencies
	 * @return void 
	 */
	public function load_dependencies(){

		/**
		 * Abstracts
		 */
		require_once SWA_ROOT .'inc/abstracts/class-swa-settings.php';

		/**
		 * Classes
		 */
		require_once SWA_ROOT .'inc/class-swa-status.php';
		
		/**
		 * Base functions
		 */
		require_once SWA_ROOT .'inc/swa-functions.php';
		require_once SWA_ROOT .'inc/swa-cpt.php';
		require_once SWA_ROOT .'inc/swa-statuses.php';

		/**
		 * Load only on admin side
		 */
		if( is_admin() ) {
			require_once SWA_ROOT .'inc/class-swa-metabox.php';
			require_once SWA_ROOT .'inc/class-swa-column.php';
			require_once SWA_ROOT .'inc/class-swa-columns.php';
		}

		/**
		 * We are on the front page
		 */
		if( ! is_admin() ) {
			require_once SWA_ROOT .'inc/swa-front.php';
		}
	}

	/**
	 * Run the plugin
	 * @return void
	 */
	public function run(){
		// Hook into the 'init' action
		add_action( 'init', 'swa_cpt', 0 );

		// Hook when swa has ended
		add_action( 'swa_ad_ended', 'swa_ad_ended' );

		// Ad click registration
		add_action( 'wp_ajax_swa_ad_click', 'swa_ad_click' );
		add_action( 'wp_ajax_nopriv_swa_ad_click', 'swa_ad_click' );

		if( is_admin() ) {
			add_action( 'load-post.php',     array( $this, 'construct_metabox_on_ad' ) );
        	add_action( 'load-post-new.php', array( $this, 'construct_metabox_on_ad' ) );
        	add_action( 'save_post', 'swa_save_ad', 99, 2 );

        	$this->make_columns();
		} 

		if( ! is_admin() ) {
			// Add Style on front end
			add_action( 'wp_head', 'swa_inline_style' );

			// Get banners
			add_action( 'init', 'swa_get_ads', 20 );

			// Add body clases used in inline styles
			add_action( 'init', 'swa_add_body_classes', 30 );

			// Display the ads
			add_action( 'wp_footer', 'swa_display_ads', 30 );

			// Add body clases used in inline styles
			add_action( 'wp_enqueue_scripts', 'swa_enqueue_scripts', 20 );
		}

	}

	public function make_columns() {
		$columns = new SWA_Columns( array(

			array(
				'post_type' => 'swa',
				'column_key' => '_impressions',
				'column_title' => __( 'Impressions', 'swa' ),
				'column_display' => 'swa_impressions'
				),
			array(
				'post_type' => 'swa',
				'column_key' => '_clicks',
				'column_title' => __( 'Clicks', 'swa' ),
				'column_display' => 'swa_clicks'
				),
			array(
				'post_type' => 'swa',
				'column_key' => '_ctr',
				'column_title' => __( 'CTR', 'swa' ),
				'column_display' => 'swa_ctr'
				),

		));

		$columns->add_columns();
	}

	/**
	 * Construct metabox(es) when on Ad edit/new post page
	 * @return void 
	 */
	public function construct_metabox_on_ad(){

		require_once SWA_ROOT .'inc/swa-metaboxes.php';

	}
}

function swa(){
	return Simple_WP_Ads::instance();
}

function swa_start(){
	$swa = swa();
	$swa->load_dependencies();
	$swa->run();
}

add_action( 'plugins_loaded', 'swa_start');